#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>

#include "forms.h"
#include "Xsoxdefs.h"

FD_Xsox    *fd_xsox;
FD_Effects *fd_effects;

static char *AudioChoices = " Use Filetype |" " Apple II (aiff) |" 
" SUN audio (au) |" " Mac (hcom) |" " Unsigned Byte (ub) |"
" Signed Byte (sb) |" " Unsigned Word (uw) |" " Signed Word (sw) |"
" U Law (ul) |" " IRCAM (sf) |" " Sound Blaster (voc) |"
" CD-format (cdr) |" " Ascii data (dat) |" " Sample Vision (smp) |"
" Windows 3.1 (wav) "
;

void SetRateDefaults(FL_OBJECT *ob)
{
   fl_set_counter_value(ob,0);
   fl_set_counter_bounds(ob,0,45000);
   fl_set_counter_step(ob,100,1000);
   fl_set_counter_precision(ob,0);
}

void Initialize(FD_Xsox *fd_xsox)
{
   int pipepair[2];
   long modes;

   SetRateDefaults(fd_xsox->InputRate);
   SetRateDefaults(fd_xsox->OutputRate);

   /* make some buttons look as if the are pushed in */   
   fl_set_button(fd_effects->Plain,1); 
   fl_set_button(fd_xsox->InputChannelOff,1);
   fl_set_button(fd_xsox->OutputChannelOff,1);

   fl_set_input(fd_xsox->PlayCommand,PLAYCMD);
   fl_addto_choice(fd_xsox->InputFormat,AudioChoices);
   fl_addto_choice(fd_xsox->OutputFormat,AudioChoices);

   /* Provide a mechanism to redirect errors from SOX to our Browser */
   pipe(pipepair);
   close(STDERR_FILENO);
   dup(pipepair[1]);       /* set input side from pipe to stderr */
   setbuf(stderr,NULL);
   
   /* force non-blocking mode so we can always read from this fd */
   modes = fcntl(pipepair[0],F_GETFL);
   fcntl(pipepair[0],F_SETFL, modes | O_NDELAY);

   /* And connect to the browser */
   fl_add_io_callback(pipepair[0],FL_READ,
		      (FL_IO_CALLBACK)AddMessages,(void *)pipepair[0]);

   fl_set_goodies_font(FL_NORMAL_TEXT,12); /* a little bigger ! */
}

int main(int argc, char *argv[])
{
   fl_initialize(&argc, argv, 0, 0, 0);
   fd_xsox    = create_form_Xsox();
   fd_effects = create_form_Effects();
   fl_show_form(fd_xsox->Xsox,FL_PLACE_ASPECT|FL_PLACE_CENTER,FL_FULLBORDER,"Xsox");

   /* set some charactersistics to provide reasonable defaults */
   Initialize(fd_xsox);
   
   /* avoid using a dummy callback for all widgets by using a loop ! */
   while(1) fl_do_forms();
   return 0;
}
